# functions that implement analysis and synthesis of sounds using the Sinusoidal Model
# (for example usage check the examples models_interface)

import math

import numpy as np
from scipy.interpolate import interp1d
from scipy.signal import blackmanharris, triang
from scipy.fftpack import ifft, fftshift

from . import dft
from ..utils import peaks, synth


def from_audio(x, fs, w, N, H, t, maxnSines=100, minSineDur=.01, freqDevOffset=20, freqDevSlope=0.01):
    """
    Analysis of a sound using the sinusoidal model with sine tracking
    x: input array sound, w: analysis window, N: size of complex spectrum, H: hop-size, t: threshold in negative dB
    maxnSines: maximum number of sines per frame, minSineDur: minimum duration of sines in seconds
    freqDevOffset: minimum frequency deviation at 0Hz, freqDevSlope: slope increase of minimum frequency deviation
    returns xtfreq, xtmag, xtphase: frequencies, magnitudes and phases of sinusoidal tracks
    """

    if (minSineDur < 0):  # raise error if minSineDur is smaller than 0
        raise ValueError("Minimum duration of sine tracks smaller than 0")

    hM1 = int(math.floor((w.size + 1) / 2))  # half analysis window size by rounding
    hM2 = int(math.floor(w.size / 2))  # half analysis window size by floor
    x = np.append(np.zeros(hM2), x)  # add zeros at beginning to center first window at sample 0
    x = np.append(x, np.zeros(hM2))  # add zeros at the end to analyze last sample
    pin = hM1  # initialize sound pointer in middle of analysis window
    pend = x.size - hM1  # last sample to start a frame
    w = w / sum(w)  # normalize analysis window
    tfreq = np.array([])
    while pin < pend:  # while input sound pointer is within sound
        x1 = x[pin - hM1:pin + hM2]  # select frame
        mX, pX = dft.from_audio(x1, w, N)  # compute dft
        ploc = peaks.find_peaks(mX, t)  # detect locations of peaks
        iploc, ipmag, ipphase = peaks.interpolate_peaks(mX, pX, ploc)  # refine peak values by interpolation
        ipfreq = fs * iploc / float(N)  # convert peak locations to Hertz
        # perform sinusoidal tracking by adding peaks to trajectories
        tfreq, tmag, tphase = track_sinusoids(ipfreq, ipmag, ipphase, tfreq, freqDevOffset, freqDevSlope)
        tfreq = np.resize(tfreq, min(maxnSines, tfreq.size))  # limit number of tracks to maxnSines
        tmag = np.resize(tmag, min(maxnSines, tmag.size))  # limit number of tracks to maxnSines
        tphase = np.resize(tphase, min(maxnSines, tphase.size))  # limit number of tracks to maxnSines
        jtfreq = np.zeros(maxnSines)  # temporary output array
        jtmag = np.zeros(maxnSines)  # temporary output array
        jtphase = np.zeros(maxnSines)  # temporary output array
        jtfreq[:tfreq.size] = tfreq  # save track frequencies to temporary array
        jtmag[:tmag.size] = tmag  # save track magnitudes to temporary array
        jtphase[:tphase.size] = tphase  # save track magnitudes to temporary array
        if pin == hM1:  # if first frame initialize output sine tracks
            xtfreq = jtfreq
            xtmag = jtmag
            xtphase = jtphase
        else:  # rest of frames append values to sine tracks
            xtfreq = np.vstack((xtfreq, jtfreq))
            xtmag = np.vstack((xtmag, jtmag))
            xtphase = np.vstack((xtphase, jtphase))
        pin += H
    # delete sine tracks shorter than minSineDur
    xtfreq = clean_sinusoid_tracks(xtfreq, round(fs * minSineDur / H))
    return xtfreq, xtmag, xtphase


def to_audio(tfreq, tmag, tphase, N, H, fs):
    """
    Synthesis of a sound using the sinusoidal model
    tfreq,tmag,tphase: frequencies, magnitudes and phases of sinusoids
    N: synthesis FFT size, H: hop size, fs: sampling rate
    returns y: output array sound
    """

    hN = N / 2  # half of FFT size for synthesis
    L = tfreq.shape[0]  # number of frames
    pout = 0  # initialize output sound pointer
    ysize = H * (L + 3)  # output sound size
    y = np.zeros(ysize)  # initialize output array
    sw = np.zeros(N)  # initialize synthesis window
    ow = triang(2 * H)  # triangular window
    sw[hN - H:hN + H] = ow  # add triangular window
    bh = blackmanharris(N)  # blackmanharris window
    bh = bh / sum(bh)  # normalized blackmanharris window
    sw[hN - H:hN + H] = sw[hN - H:hN + H] / bh[hN - H:hN + H]  # normalized synthesis window
    lastytfreq = tfreq[0, :]  # initialize synthesis frequencies
    ytphase = 2 * np.pi * np.random.rand(tfreq[0, :].size)  # initialize synthesis phases
    for l in range(L):  # iterate over all frames
        if tphase.size > 0:  # if no phases generate them
            ytphase = tphase[l, :]
        else:
            ytphase += (np.pi * (lastytfreq + tfreq[l, :]) / fs) * H  # propagate phases
        Y = synth.spectrum_for_sinusoids(tfreq[l, :], tmag[l, :], ytphase, N, fs)  # generate sines in the spectrum
        lastytfreq = tfreq[l, :]  # save frequency for phase propagation
        ytphase %= 2 * np.pi  # make phase inside 2*pi
        yw = np.real(fftshift(ifft(Y)))  # compute inverse FFT
        y[pout:pout + N] += sw * yw  # overlap-add and apply a synthesis window
        pout += H  # advance sound pointer
    y = np.delete(y, range(hN))  # delete half of first window
    y = np.delete(y, range(y.size - hN, y.size))  # delete half of the last window
    return y


def reconstruct(x, fs, w, N, t):
    """
    Analysis/synthesis of a sound using the sinusoidal model, without sine tracking
    x: input array sound, w: analysis window, N: size of complex spectrum, t: threshold in negative dB
    returns y: output array sound
    """

    hM1 = int(math.floor((w.size + 1) / 2))  # half analysis window size by rounding
    hM2 = int(math.floor(w.size / 2))  # half analysis window size by floor
    Ns = 512  # FFT size for synthesis (even)
    H = Ns / 4  # Hop size used for analysis and synthesis
    hNs = Ns / 2  # half of synthesis FFT size
    pin = max(hNs, hM1)  # init sound pointer in middle of anal window
    pend = x.size - max(hNs, hM1)  # last sample to start a frame
    fftbuffer = np.zeros(N)  # initialize buffer for FFT
    yw = np.zeros(Ns)  # initialize output sound frame
    y = np.zeros(x.size)  # initialize output array
    w = w / sum(w)  # normalize analysis window
    sw = np.zeros(Ns)  # initialize synthesis window
    ow = triang(2 * H)  # triangular window
    sw[hNs - H:hNs + H] = ow  # add triangular window
    bh = blackmanharris(Ns)  # blackmanharris window
    bh = bh / sum(bh)  # normalized blackmanharris window
    sw[hNs - H:hNs + H] = sw[hNs - H:hNs + H] / bh[hNs - H:hNs + H]  # normalized synthesis window
    while pin < pend:  # while input sound pointer is within sound
        # -----analysis-----
        x1 = x[pin - hM1:pin + hM2]  # select frame
        mX, pX = dft.from_audio(x1, w, N)  # compute dft
        ploc = peaks.find_peaks(mX, t)  # detect locations of peaks
        iploc, ipmag, ipphase = peaks.interpolate_peaks(mX, pX, ploc)  # refine peak values by interpolation
        ipfreq = fs * iploc / float(N)  # convert peak locations to Hertz
        # -----synthesis-----
        Y = synth.spectrum_for_sinusoids(ipfreq, ipmag, ipphase, Ns, fs)  # generate sines in the spectrum
        fftbuffer = np.real(ifft(Y))  # compute inverse FFT
        yw[:hNs - 1] = fftbuffer[hNs + 1:]  # undo zero-phase window
        yw[hNs - 1:] = fftbuffer[:hNs + 1]
        y[pin - hNs:pin + hNs] += sw * yw  # overlap-add and apply a synthesis window
        pin += H  # advance sound pointer
    return y


# functions that implement transformations using the sineModel

def scale_time(sfreq, smag, timeScaling):
    """
    Time scaling of sinusoidal tracks
    sfreq, smag: frequencies and magnitudes of input sinusoidal tracks
    timeScaling: scaling factors, in time-value pairs
    returns ysfreq, ysmag: frequencies and magnitudes of output sinusoidal tracks
    """
    if timeScaling.size % 2 != 0:  # raise exception if array not even length
        raise ValueError("Time scaling array does not have an even size")

    L = sfreq.shape[0]  # number of input frames
    maxInTime = max(timeScaling[::2])  # maximum value used as input times
    maxOutTime = max(timeScaling[1::2])  # maximum value used in output times
    outL = int(L * maxOutTime / maxInTime)  # number of output frames
    inFrames = (L - 1) * timeScaling[::2] / maxInTime  # input time values in frames
    outFrames = outL * timeScaling[1::2] / maxOutTime  # output time values in frames
    timeScalingEnv = interp1d(outFrames, inFrames, fill_value=0)  # interpolation function
    indexes = timeScalingEnv(np.arange(outL))  # generate frame indexes for the output
    ysfreq = sfreq[round(indexes[0]), :]  # first output frame
    ysmag = smag[round(indexes[0]), :]  # first output frame
    for l in indexes[1:]:  # generate frames for output sine tracks
        ysfreq = np.vstack((ysfreq, sfreq[round(l), :]))  # get closest frame to scaling value
        ysmag = np.vstack((ysmag, smag[round(l), :]))  # get closest frame to scaling value
    return ysfreq, ysmag


def scale_frequencies(sfreq, freqScaling):
    """
    Frequency scaling of sinusoidal tracks
    sfreq: frequencies of input sinusoidal tracks
    freqScaling: scaling factors, in time-value pairs (value of 1 is no scaling)
    returns ysfreq: frequencies of output sinusoidal tracks
    """
    if (freqScaling.size % 2 != 0):  # raise exception if array not even length
        raise ValueError("Frequency scaling array does not have an even size")

    L = sfreq.shape[0]  # number of input frames
    # create interpolation object from the scaling values
    freqScalingEnv = np.interp(np.arange(L), L * freqScaling[::2] / freqScaling[-2], freqScaling[1::2])
    ysfreq = np.zeros_like(sfreq)  # create empty output matrix
    for l in range(L):  # go through all frames
        ind_valid = np.where(sfreq[l, :] != 0)[0]  # check if there are frequency values
        if ind_valid.size == 0:  # if no values go to next frame
            continue
        ysfreq[l, ind_valid] = sfreq[l, ind_valid] * freqScalingEnv[l]  # scale of frequencies
    return ysfreq


# -- support functions --

def track_sinusoids(pfreq, pmag, pphase, tfreq, freqDevOffset=20, freqDevSlope=0.01):
    """
    Tracking sinusoids from one frame to the next
    pfreq, pmag, pphase: frequencies and magnitude of current frame
    tfreq: frequencies of incoming tracks from previous frame
    freqDevOffset: minimum frequency deviation at 0Hz
    freqDevSlope: slope increase of minimum frequency deviation
    returns tfreqn, tmagn, tphasen: frequency, magnitude and phase of tracks
    """

    tfreqn = np.zeros(tfreq.size)  # initialize array for output frequencies
    tmagn = np.zeros(tfreq.size)  # initialize array for output magnitudes
    tphasen = np.zeros(tfreq.size)  # initialize array for output phases
    pindexes = np.array(np.nonzero(pfreq), dtype=np.int)[0]  # indexes of current peaks
    incomingTracks = np.array(np.nonzero(tfreq), dtype=np.int)[0]  # indexes of incoming tracks
    newTracks = np.zeros(tfreq.size, dtype=np.int) - 1  # initialize to -1 new tracks
    magOrder = np.argsort(-pmag[pindexes])  # order current peaks by magnitude
    pfreqt = np.copy(pfreq)  # copy current peaks to temporary array
    pmagt = np.copy(pmag)  # copy current peaks to temporary array
    pphaset = np.copy(pphase)  # copy current peaks to temporary array

    # continue incoming tracks
    if incomingTracks.size > 0:  # if incoming tracks exist
        for i in magOrder:  # iterate over current peaks
            if incomingTracks.size == 0:  # break when no more incoming tracks
                break
            track = np.argmin(abs(pfreqt[i] - tfreq[incomingTracks]))  # closest incoming track to peak
            freqDistance = abs(pfreq[i] - tfreq[incomingTracks[track]])  # measure freq distance
            if freqDistance < (freqDevOffset + freqDevSlope * pfreq[i]):  # choose track if distance is small
                newTracks[incomingTracks[track]] = i  # assign peak index to track index
                incomingTracks = np.delete(incomingTracks, track)  # delete index of track in incoming tracks
    indext = np.array(np.nonzero(newTracks != -1), dtype=np.int)[0]  # indexes of assigned tracks
    if indext.size > 0:
        indexp = newTracks[indext]  # indexes of assigned peaks
        tfreqn[indext] = pfreqt[indexp]  # output freq tracks
        tmagn[indext] = pmagt[indexp]  # output mag tracks
        tphasen[indext] = pphaset[indexp]  # output phase tracks
        pfreqt = np.delete(pfreqt, indexp)  # delete used peaks
        pmagt = np.delete(pmagt, indexp)  # delete used peaks
        pphaset = np.delete(pphaset, indexp)  # delete used peaks

    # create new tracks from non used peaks
    emptyt = np.array(np.nonzero(tfreq == 0), dtype=np.int)[0]  # indexes of empty incoming tracks
    peaksleft = np.argsort(-pmagt)  # sort left peaks by magnitude
    if ((peaksleft.size > 0) & (emptyt.size >= peaksleft.size)):  # fill empty tracks
        tfreqn[emptyt[:peaksleft.size]] = pfreqt[peaksleft]
        tmagn[emptyt[:peaksleft.size]] = pmagt[peaksleft]
        tphasen[emptyt[:peaksleft.size]] = pphaset[peaksleft]
    elif ((peaksleft.size > 0) & (emptyt.size < peaksleft.size)):  # add more tracks if necessary
        tfreqn[emptyt] = pfreqt[peaksleft[:emptyt.size]]
        tmagn[emptyt] = pmagt[peaksleft[:emptyt.size]]
        tphasen[emptyt] = pphaset[peaksleft[:emptyt.size]]
        tfreqn = np.append(tfreqn, pfreqt[peaksleft[emptyt.size:]])
        tmagn = np.append(tmagn, pmagt[peaksleft[emptyt.size:]])
        tphasen = np.append(tphasen, pphaset[peaksleft[emptyt.size:]])
    return tfreqn, tmagn, tphasen


def clean_sinusoid_tracks(tfreq, minTrackLength=3):
    """
    Delete short fragments of a collection of sinusoidal tracks
    tfreq: frequency of tracks
    minTrackLength: minimum duration of tracks in number of frames
    returns tfreqn: output frequency of tracks
    """

    if tfreq.shape[1] == 0:  # if no tracks return input
        return tfreq
    nFrames = tfreq[:, 0].size  # number of frames
    nTracks = tfreq[0, :].size  # number of tracks in a frame
    for t in range(nTracks):  # iterate over all tracks
        trackFreqs = tfreq[:, t]  # frequencies of one track
        trackBegs = np.nonzero((trackFreqs[:nFrames - 1] <= 0)  # beginning of track contours
                               & (trackFreqs[1:] > 0))[0] + 1
        if trackFreqs[0] > 0:
            trackBegs = np.insert(trackBegs, 0, 0)
        trackEnds = np.nonzero((trackFreqs[:nFrames - 1] > 0)  # end of track contours
                               & (trackFreqs[1:] <= 0))[0] + 1
        if trackFreqs[nFrames - 1] > 0:
            trackEnds = np.append(trackEnds, nFrames - 1)
        trackLengths = 1 + trackEnds - trackBegs  # lengths of track contours
        for i, j in zip(trackBegs, trackLengths):  # delete short track contours
            if j <= minTrackLength:
                trackFreqs[i:i + j] = 0
    return tfreq



# function to call the main analysis/synthesis functions in software/models/stochasticModel.py

import os

import matplotlib.pyplot as plt
import numpy as np

from smst.utils import audio, files
from smst.models import stochastic
from .. import demo_sound_path


def main(inputFile=demo_sound_path('ocean.wav'), H=256, N=512, stocf=.1,
         interactive=True, plotFile=False):
    """
    inputFile: input sound file (monophonic with sampling rate of 44100)
    H: hop size, N: fft size
    stocf: decimation factor used for the stochastic approximation (bigger than 0, maximum 1)
    """

    # read input sound
    (fs, x) = audio.read_wav(inputFile)

    # compute stochastic model
    stocEnv = stochastic.from_audio(x, H, N, stocf)

    # synthesize sound from stochastic model
    y = stochastic.to_audio(stocEnv, H, N)

    outputFile = 'output_sounds/' + os.path.basename(inputFile)[:-4] + '_stochasticModel.wav'

    # write output sound
    audio.write_wav(y, fs, outputFile)

    # create figure to plot
    plt.figure(figsize=(12, 9))

    # plot the input sound
    plt.subplot(3, 1, 1)
    plt.plot(np.arange(x.size) / float(fs), x)
    plt.axis([0, x.size / float(fs), min(x), max(x)])
    plt.ylabel('amplitude')
    plt.xlabel('time (sec)')
    plt.title('input sound: x')

    # plot stochastic representation
    plt.subplot(3, 1, 2)
    numFrames = int(stocEnv[:, 0].size)
    frmTime = H * np.arange(numFrames) / float(fs)
    binFreq = np.arange(stocf * (N / 2 + 1)) * float(fs) / (stocf * N)
    plt.pcolormesh(frmTime, binFreq, np.transpose(stocEnv))
    plt.autoscale(tight=True)
    plt.xlabel('time (sec)')
    plt.ylabel('frequency (Hz)')
    plt.title('stochastic approximation')

    # plot the output sound
    plt.subplot(3, 1, 3)
    plt.plot(np.arange(y.size) / float(fs), y)
    plt.axis([0, y.size / float(fs), min(y), max(y)])
    plt.ylabel('amplitude')
    plt.xlabel('time (sec)')

    plt.tight_layout()

    if interactive:
        plt.show()
    if plotFile:
        plt.savefig('output_plots/%s_stochastic_model.png' % files.strip_file(inputFile))


if __name__ == "__main__":
    main()



import copy
import os
import subprocess
import sys

import numpy as np
from scipy.io.wavfile import write, read

from .files import ensure_directory

winsound_imported = False
if sys.platform == "win32":
    try:
        import winsound

        winsound_imported = True
    except:
        print "You won't be able to play sounds, winsound could not be imported"

INT16_FAC = (2 ** 15) - 1
INT32_FAC = (2 ** 31) - 1
INT64_FAC = (2 ** 63) - 1
norm_fact = {'int16': INT16_FAC, 'int32': INT32_FAC, 'int64': INT64_FAC, 'float32': 1.0, 'float64': 1.0}


def read_wav(filename):
    """
    Read a sound file and convert it to a normalized floating point array
    filename: name of file to read
    returns fs: sampling rate of file, x: floating point array
    """

    if not os.path.isfile(filename):  # raise error if wrong input file
        raise ValueError("Input file is wrong")

    fs, x = read(filename)

    if len(x.shape) != 1:  # raise error if more than one channel
        raise ValueError("Audio file should be mono")

    if fs != 44100:  # raise error if more than one channel
        raise ValueError("Sampling rate of input sound should be 44100")

    # scale down and convert audio into floating point number in range of -1 to 1
    x = np.float32(x) / norm_fact[x.dtype.name]
    return fs, x


def play_wav(filename):
    """
    Play a wav audio file from system using OS calls
    filename: name of file to read
    """
    if not os.path.isfile(filename):  # raise error if wrong input file
        print("Input file does not exist. Make sure you computed the analysis/synthesis")
    else:
        if sys.platform == "linux" or sys.platform == "linux2":
            # linux
            subprocess.call(["aplay", filename])

        elif sys.platform == "darwin":
            # OS X
            subprocess.call(["afplay", filename])
        elif sys.platform == "win32":
            if winsound_imported:
                winsound.PlaySound(filename, winsound.SND_FILENAME)
            else:
                print("Cannot play sound, winsound could not be imported")
        else:
            print("Platform not recognized")


def write_wav(y, fs, filename):
    """
    Write a sound file from an array with the sound and the sampling rate.
    Creates the directory for the file if it does not exist.
    y: floating point array of one dimension, fs: sampling rate
    filename: name of file to create (can be a path)
    """

    dir = os.path.dirname(filename)
    ensure_directory(dir)

    x = copy.deepcopy(y)  # copy array
    x *= INT16_FAC  # scaling floating point -1 to 1 range signal to int16 range
    x = np.int16(x)  # converting to int16 type
    write(filename, fs, x)



from __future__ import print_function
import os
import sys
import importlib
import pkgutil

_sound_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../../sounds')

def demo_sound_path(file_name):
    'locates a demo sound file'
    return os.path.join(_sound_dir, file_name)

# this allow to run an example of each model or transformation
# eg. $ smst-model dftModel

def run_module(parent_module_name):
    def list_modules(parent_module_path):
        return [module_name for (_, module_name, _) in pkgutil.iter_modules(parent_module_path)]

    module = sys.argv[1] if len(sys.argv) > 1 else None
    parent_module = importlib.import_module(parent_module_name)
    modules = [m.replace('_function', '') for m in
        list_modules(parent_module.__path__)
        if m.endswith('_function')]
    if module in modules:
        importlib.import_module('%s.%s_function' % (parent_module_name, module)).main()
    else:
        print('Available modules:')
        for module in modules:
            print('-', module)

def main_model():
    run_module('smst.ui.models')

def main_transformation():
    run_module('smst.ui.transformations')



"""SMS tools installation script.

Packaging docs: https://packaging.python.org/en/latest/distributing.html
setup.py adapted from https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# the the cython extension
from setuptools import Extension
from Cython.Distutils import build_ext
import numpy as np

# To use a consistent encoding
from codecs import open

from os import path
import re

# here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
# with open(path.join(here, 'README.md'), encoding='utf-8') as f:
#     long_description = f.read()

# Cython extensions
sourcefiles = [
    'smst/utils/utilFunctions_C/utilFunctions.c',
    'smst/utils/utilFunctions_C/cutilFunctions.pyx'
]
ext_modules = [Extension(
    'smst.utils.utilFunctions_C.utilFunctions_C',
    sourcefiles,
    libraries=['m'],
    # include_dirs=py_inc + np_inc
    include_dirs=[np.get_include()]
)]

def read_version():
    """
    Reads the project version without importing the module.
    This prevent importing a wrong module and importing the module before its
    dependencies are built.
    """
    version = ''
    with open('smst/__init__.py', 'r') as file:
        version = re.search(
            r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
            file.read(),
            re.MULTILINE
        ).group(1)
    if not version:
        raise RuntimeError('Could not find package version from __init__.py')
    return version

setup(
    name='smst',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=read_version(),

    description='SMS tools - spectral audio modeling/synthesis/transformations',
    # long_description=long_description,

    # The project's main homepage.
    url='https://github.com/MTG/sms-tools',

    # Author details
    # See the AUTHORS file for the complete list of authors
    author='Xavier Serra, Fabia Serra Arrizabalaga, Sankalp Gulati and others',
    author_email='xavier.serra@upf.edu',

    # Choose your license
    license='AGPL',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Topic :: Multimedia :: Sound/Audio :: Analysis',
        'Topic :: Multimedia :: Sound/Audio :: Sound Synthesis',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',

        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
    ],

    # What does your project relate to?
    keywords='audio spectral modelling analysis synthesis',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    # py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['numpy', 'scipy', 'matplotlib', 'cython'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    # extras_require={
    #     'dev': ['check-manifest'],
    #     'test': ['coverage'],
    # },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # package_data={
    #     '': ['AUTHORS', 'LICENSE', 'README.md']
    # },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'smst-ui-models=smst.ui.models.models_GUI:main',
            'smst-ui-transformations=smst.ui.transformations.transformations_GUI:main',
            'smst-model=smst.ui:main_model',
            'smst-transformation=smst.ui:main_transformation',
        ],
    },

    # Cython extensions
    cmdclass = {'build_ext': build_ext},
    ext_modules = ext_modules

)



import os


def strip_file(filePath):
    """extracts file name without directories and extension"""
    return os.path.splitext(os.path.basename(filePath))[0]


def ensure_directory(dir):
    if len(dir) > 0 and not os.path.isdir(dir) and not os.path.isfile(dir):
        os.makedirs(dir)



# GUI frame for the stochasticTransformations_function.py

import os
from Tkinter import *
import tkFileDialog
import tkMessageBox

import numpy as np

from smst.utils import audio
from . import stochasticTransformations_function as sT


class StochasticTransformationsFrame:
    def __init__(self, parent):

        self.parent = parent
        self.initUI()

    def initUI(self):

        choose_label = "inputFile:"
        Label(self.parent, text=choose_label).grid(row=0, column=0, sticky=W, padx=5, pady=(10, 2))

        # TEXTBOX TO PRINT PATH OF THE SOUND FILE
        self.filelocation = Entry(self.parent)
        self.filelocation.focus_set()
        self.filelocation["width"] = 25
        self.filelocation.grid(row=0, column=0, sticky=W, padx=(70, 5), pady=(10, 2))
        self.filelocation.delete(0, END)
        self.filelocation.insert(0, 'sounds/rain.wav')

        # BUTTON TO BROWSE SOUND FILE
        open_file = Button(self.parent, text="...", command=self.browse_file)  # see: def browse_file(self)
        open_file.grid(row=0, column=0, sticky=W, padx=(280, 6), pady=(10, 2))  # put it beside the filelocation textbox

        # BUTTON TO PREVIEW SOUND FILE
        preview = Button(self.parent, text=">", command=lambda: audio.play_wav(self.filelocation.get()), bg="gray30",
                         fg="white")
        preview.grid(row=0, column=0, sticky=W, padx=(325, 6), pady=(10, 2))

        ## STOCHASTIC TRANSFORMATIONS ANALYSIS

        # DECIMATION FACTOR
        stocf_label = "stocf:"
        Label(self.parent, text=stocf_label).grid(row=1, column=0, sticky=W, padx=(5, 5), pady=(10, 2))
        self.stocf = Entry(self.parent, justify=CENTER)
        self.stocf["width"] = 5
        self.stocf.grid(row=1, column=0, sticky=W, padx=(47, 5), pady=(10, 2))
        self.stocf.delete(0, END)
        self.stocf.insert(0, "0.1")

        # TIME SCALING FACTORS
        timeScaling_label = "Time scaling factors (time, value pairs):"
        Label(self.parent, text=timeScaling_label).grid(row=2, column=0, sticky=W, padx=5, pady=(5, 2))
        self.timeScaling = Entry(self.parent, justify=CENTER)
        self.timeScaling["width"] = 35
        self.timeScaling.grid(row=3, column=0, sticky=W + E, padx=5, pady=(0, 2))
        self.timeScaling.delete(0, END)
        self.timeScaling.insert(0, "[0, 0, 1, 2]")

        # BUTTON TO DO THE SYNTHESIS
        self.compute = Button(self.parent, text="Apply Transformation", command=self.transformation_synthesis,
                              bg="dark green", fg="white")
        self.compute.grid(row=13, column=0, padx=5, pady=(10, 15), sticky=W)

        # BUTTON TO PLAY TRANSFORMATION SYNTHESIS OUTPUT
        self.transf_output = Button(self.parent, text=">", command=lambda: audio.play_wav(
            'output_sounds/' + os.path.basename(self.filelocation.get())[:-4] + '_stochasticModelTransformation.wav'),
                                    bg="gray30", fg="white")
        self.transf_output.grid(row=13, column=0, padx=(165, 5), pady=(10, 15), sticky=W)

        # define options for opening file
        self.file_opt = options = {}
        options['defaultextension'] = '.wav'
        options['filetypes'] = [('All files', '.*'), ('Wav files', '.wav')]
        options['initialdir'] = 'sounds/'
        options['title'] = 'Open a mono audio file .wav with sample frequency 44100 Hz'

    def browse_file(self):

        self.filename = tkFileDialog.askopenfilename(**self.file_opt)

        # set the text of the self.filelocation
        self.filelocation.delete(0, END)
        self.filelocation.insert(0, self.filename)

    def transformation_synthesis(self):

        try:
            inputFile = self.filelocation.get()
            stocf = float(self.stocf.get())
            timeScaling = np.array(eval(self.timeScaling.get()))

            sT.main(inputFile, stocf, timeScaling)

        except ValueError as errorMessage:
            tkMessageBox.showerror("Input values error", errorMessage)



# function to call the main analysis/synthesis functions in software/models/sineModel.py

import os

import matplotlib.pyplot as plt
import numpy as np
from scipy.signal import get_window

from smst.utils import audio, files
from smst.models import sine
from .. import demo_sound_path


def main(inputFile=demo_sound_path('bendir.wav'), window='hamming', M=2001, N=2048, t=-80, minSineDur=0.02,
         maxnSines=150, freqDevOffset=10, freqDevSlope=0.001,
         interactive=True, plotFile=False):
    """
    Perform analysis/synthesis using the sinusoidal model
    inputFile: input sound file (monophonic with sampling rate of 44100)
    window: analysis window type (rectangular, hanning, hamming, blackman, blackmanharris)
    M: analysis window size; N: fft size (power of two, bigger or equal than M)
    t: magnitude threshold of spectral peaks; minSineDur: minimum duration of sinusoidal tracks
    maxnSines: maximum number of parallel sinusoids
    freqDevOffset: frequency deviation allowed in the sinusoids from frame to frame at frequency 0
    freqDevSlope: slope of the frequency deviation, higher frequencies have bigger deviation
    """

    # size of fft used in synthesis
    Ns = 512

    # hop size (has to be 1/4 of Ns)
    H = 128

    # read input sound
    fs, x = audio.read_wav(inputFile)

    # compute analysis window
    w = get_window(window, M)

    # analyze the sound with the sinusoidal model
    tfreq, tmag, tphase = sine.from_audio(x, fs, w, N, H, t, maxnSines, minSineDur, freqDevOffset, freqDevSlope)

    # synthesize the output sound from the sinusoidal representation
    y = sine.to_audio(tfreq, tmag, tphase, Ns, H, fs)

    # output sound file name
    outputFile = 'output_sounds/' + os.path.basename(inputFile)[:-4] + '_sineModel.wav'

    # write the synthesized sound obtained from the sinusoidal synthesis
    audio.write_wav(y, fs, outputFile)

    # create figure to show plots
    plt.figure(figsize=(12, 9))

    # frequency range to plot
    maxplotfreq = 5000.0

    # plot the input sound
    plt.subplot(3, 1, 1)
    plt.plot(np.arange(x.size) / float(fs), x)
    plt.axis([0, x.size / float(fs), min(x), max(x)])
    plt.ylabel('amplitude')
    plt.xlabel('time (sec)')
    plt.title('input sound: x')

    # plot the sinusoidal frequencies
    plt.subplot(3, 1, 2)
    if (tfreq.shape[1] > 0):
        numFrames = tfreq.shape[0]
        frmTime = H * np.arange(numFrames) / float(fs)
        tfreq[tfreq <= 0] = np.nan
        plt.plot(frmTime, tfreq)
        plt.axis([0, x.size / float(fs), 0, maxplotfreq])
        plt.title('frequencies of sinusoidal tracks')

    # plot the output sound
    plt.subplot(3, 1, 3)
    plt.plot(np.arange(y.size) / float(fs), y)
    plt.axis([0, y.size / float(fs), min(y), max(y)])
    plt.ylabel('amplitude')
    plt.xlabel('time (sec)')
    plt.title('output sound: y')

    plt.tight_layout()

    if interactive:
        plt.show()
    if plotFile:
        plt.savefig('output_plots/%s_sine_model.png' % files.strip_file(inputFile))


if __name__ == "__main__":
    main()



#this is a cython wrapper on C functions to call them in python

import numpy as np
cimport numpy as np
from libc.stdlib cimport *
from cutilFunctions cimport *

np.import_array()


def twm(pfreq, pmag, f0c):
    """This is a cython wrapper for a C function which is bit exact with the python version of this function
       For information about the input arguments please refere to the original python function
    """
    
    
    cdef np.ndarray[np.float_t, ndim=1] f0_arr
    cdef np.ndarray[np.float_t, ndim=1] f0Error_arr
    cdef np.ndarray[np.float_t, ndim=1] pfreq_arr
    cdef np.ndarray[np.float_t, ndim=1] pmag_arr
    cdef np.ndarray[np.float_t, ndim=1] f0c_arr
    
    f0_arr = np.ascontiguousarray(np.array([-1]), dtype=np.float)
    f0Error_arr = np.ascontiguousarray(np.array([-1]), dtype=np.float)

    pfreq_arr = np.ascontiguousarray(pfreq, dtype=np.float)
    pmag_arr = np.ascontiguousarray(pmag, dtype=np.float)
    f0c_arr = np.ascontiguousarray(f0c, dtype=np.float)

    TWM_C(<double*>pfreq_arr.data, <double *>pmag_arr.data, pfreq_arr.shape[0], <double *>f0c_arr.data, f0c_arr.shape[0], <double*>f0_arr.data, <double*>f0Error_arr.data)

    return f0_arr[0], f0Error_arr[0]


def genbh92lobe(x):
    "comments"
    
    cdef np.ndarray[np.float_t, ndim=1] x_arr
    cdef np.ndarray[np.float_t, ndim=1] y_arr
    
    
    x_arr = np.ascontiguousarray(x, dtype=np.float)
    y_arr = np.empty((x_arr.shape[0],), dtype=np.float)
    
    genbh92lobe_C(<double *>x_arr.data,<double *>y_arr.data, x_arr.shape[0])

    
    return y_arr
    
    
    
def genSpecSines(iploc, ipmag, ipphase,N):
    "comments"
    
    cdef np.ndarray[np.float_t, ndim=1] iploc_arr
    cdef np.ndarray[np.float_t, ndim=1] ipmag_arr
    cdef np.ndarray[np.float_t, ndim=1] ipphase_arr
    cdef np.ndarray[np.float_t, ndim=1] real_arr
    cdef np.ndarray[np.float_t, ndim=1] imag_arr
        
    iploc_arr = np.ascontiguousarray(iploc, dtype=np.float)
    ipmag_arr = np.ascontiguousarray(ipmag, dtype=np.float)
    ipphase_arr = np.ascontiguousarray(ipphase, dtype=np.float)
    
    real_arr = np.zeros((N,), dtype=np.float)
    imag_arr = np.zeros((N,), dtype=np.float)
        
    genspecsines_C(<double *>iploc_arr.data, <double *>ipmag_arr.data, <double *>ipphase_arr.data, iploc_arr.shape[0],  <double *>real_arr.data,  <double *>imag_arr.data, N)
    
    out = real_arr.astype(complex)
    out.imag = imag_arr
    return out



# GUI frame for the stftMorph_function.py

import os
from Tkinter import *
import tkFileDialog
import tkMessageBox

from smst.utils import audio
from . import stftMorph_function as sT


class StftMorphFrame:
    def __init__(self, parent):

        self.parent = parent
        self.initUI()

    def initUI(self):

        ## INPUT FILE 1
        choose1_label = "inputFile1:"
        Label(self.parent, text=choose1_label).grid(row=0, column=0, sticky=W, padx=5, pady=(10, 2))

        # TEXTBOX TO PRINT PATH OF THE SOUND FILE
        self.filelocation1 = Entry(self.parent)
        self.filelocation1.focus_set()
        self.filelocation1["width"] = 30
        self.filelocation1.grid(row=0, column=0, sticky=W, padx=(75, 5), pady=(10, 2))
        self.filelocation1.delete(0, END)
        self.filelocation1.insert(0, 'sounds/ocean.wav')

        # BUTTON TO BROWSE SOUND FILE 1
        open_file1 = Button(self.parent, text="...", command=self.browse_file1)  # see: def browse_file(self)
        open_file1.grid(row=0, column=0, sticky=W, padx=(330, 6),
                        pady=(10, 2))  # put it beside the filelocation textbox

        # BUTTON TO PREVIEW SOUND FILE 1
        preview1 = Button(self.parent, text=">", command=lambda: audio.play_wav(self.filelocation1.get()), bg="gray30",
                          fg="white")
        preview1.grid(row=0, column=0, sticky=W, padx=(375, 6), pady=(10, 2))

        # ANALYSIS WINDOW TYPE SOUND 1
        wtype1_label = "window1:"
        Label(self.parent, text=wtype1_label).grid(row=1, column=0, sticky=W, padx=5, pady=(4, 2))
        self.w1_type = StringVar()
        self.w1_type.set("hamming")  # initial value
        window1_option = OptionMenu(self.parent, self.w1_type, "rectangular", "hanning", "hamming", "blackman",
                                    "blackmanharris")
        window1_option.grid(row=1, column=0, sticky=W, padx=(68, 5), pady=(4, 2))

        # WINDOW SIZE SOUND 1
        M1_label = "M1:"
        Label(self.parent, text=M1_label).grid(row=1, column=0, sticky=W, padx=(180, 5), pady=(4, 2))
        self.M1 = Entry(self.parent, justify=CENTER)
        self.M1["width"] = 5
        self.M1.grid(row=1, column=0, sticky=W, padx=(208, 5), pady=(4, 2))
        self.M1.delete(0, END)
        self.M1.insert(0, "1024")

        # FFT SIZE SOUND 1
        N1_label = "N1:"
        Label(self.parent, text=N1_label).grid(row=1, column=0, sticky=W, padx=(265, 5), pady=(4, 2))
        self.N1 = Entry(self.parent, justify=CENTER)
        self.N1["width"] = 5
        self.N1.grid(row=1, column=0, sticky=W, padx=(290, 5), pady=(4, 2))
        self.N1.delete(0, END)
        self.N1.insert(0, "1024")

        # HOP SIZE SOUND 1
        H1_label = "H1:"
        Label(self.parent, text=H1_label).grid(row=1, column=0, sticky=W, padx=(343, 5), pady=(4, 2))
        self.H1 = Entry(self.parent, justify=CENTER)
        self.H1["width"] = 5
        self.H1.grid(row=1, column=0, sticky=W, padx=(370, 5), pady=(4, 2))
        self.H1.delete(0, END)
        self.H1.insert(0, "256")

        ###
        # SEPARATION LINE
        Frame(self.parent, height=1, width=50, bg="black").grid(row=2, pady=15, sticky=W + E)
        ###

        ## INPUT FILE 2
        choose2_label = "inputFile2:"
        Label(self.parent, text=choose2_label).grid(row=3, column=0, sticky=W, padx=5, pady=(2, 2))

        # TEXTBOX TO PRINT PATH OF THE SOUND FILE
        self.filelocation2 = Entry(self.parent)
        self.filelocation2.focus_set()
        self.filelocation2["width"] = 30
        self.filelocation2.grid(row=3, column=0, sticky=W, padx=(75, 5), pady=(2, 2))
        self.filelocation2.delete(0, END)
        self.filelocation2.insert(0, 'sounds/speech-male.wav')

        # BUTTON TO BROWSE SOUND FILE 2
        open_file2 = Button(self.parent, text="...", command=self.browse_file2)  # see: def browse_file(self)
        open_file2.grid(row=3, column=0, sticky=W, padx=(330, 6), pady=(2, 2))  # put it beside the filelocation textbox

        # BUTTON TO PREVIEW SOUND FILE 2
        preview2 = Button(self.parent, text=">", command=lambda: audio.play_wav(self.filelocation2.get()), bg="gray30",
                          fg="white")
        preview2.grid(row=3, column=0, sticky=W, padx=(375, 6), pady=(2, 2))


        # ANALYSIS WINDOW TYPE SOUND 2
        wtype2_label = "window2:"
        Label(self.parent, text=wtype2_label).grid(row=4, column=0, sticky=W, padx=5, pady=(4, 2))
        self.w2_type = StringVar()
        self.w2_type.set("hamming")  # initial value
        window2_option = OptionMenu(self.parent, self.w2_type, "rectangular", "hanning", "hamming", "blackman",
                                    "blackmanharris")
        window2_option.grid(row=4, column=0, sticky=W, padx=(68, 5), pady=(4, 2))

        # WINDOW SIZE SOUND 2
        M2_label = "M2:"
        Label(self.parent, text=M2_label).grid(row=4, column=0, sticky=W, padx=(180, 5), pady=(4, 2))
        self.M2 = Entry(self.parent, justify=CENTER)
        self.M2["width"] = 5
        self.M2.grid(row=4, column=0, sticky=W, padx=(208, 5), pady=(4, 2))
        self.M2.delete(0, END)
        self.M2.insert(0, "1024")

        # FFT SIZE SOUND 2
        N2_label = "N2:"
        Label(self.parent, text=N2_label).grid(row=4, column=0, sticky=W, padx=(265, 5), pady=(4, 2))
        self.N2 = Entry(self.parent, justify=CENTER)
        self.N2["width"] = 5
        self.N2.grid(row=4, column=0, sticky=W, padx=(290, 5), pady=(4, 2))
        self.N2.delete(0, END)
        self.N2.insert(0, "1024")

        ###
        # SEPARATION LINE
        Frame(self.parent, height=1, width=50, bg="black").grid(row=5, pady=15, sticky=W + E)
        ###

        # SMOOTHING FACTOR
        smoothf_label1 = "Smooth factor of sound 2 (bigger than 0 to max of 1, where 1 is no"
        Label(self.parent, text=smoothf_label1).grid(row=6, column=0, sticky=W, padx=(5, 5), pady=(2, 2))
        smoothf_label2 = "smothing):"
        Label(self.parent, text=smoothf_label2).grid(row=7, column=0, sticky=W, padx=(5, 5), pady=(0, 2))
        self.smoothf = Entry(self.parent, justify=CENTER)
        self.smoothf["width"] = 5
        self.smoothf.grid(row=8, column=0, sticky=W, padx=(5, 5), pady=(2, 2))
        self.smoothf.delete(0, END)
        self.smoothf.insert(0, "0.5")

        # BALANCE FACTOR
        balancef_label = "Balance factor (from 0 to 1, where 0 is sound 1 and 1 is sound 2):"
        Label(self.parent, text=balancef_label).grid(row=9, column=0, sticky=W, padx=(5, 5), pady=(10, 2))
        self.balancef = Entry(self.parent, justify=CENTER)
        self.balancef["width"] = 5
        self.balancef.grid(row=10, column=0, sticky=W, padx=(5, 5), pady=(2, 2))
        self.balancef.delete(0, END)
        self.balancef.insert(0, "0.2")

        # BUTTON TO DO THE SYNTHESIS
        self.compute = Button(self.parent, text="Apply Transformation", command=self.transformation_synthesis,
                              bg="dark green", fg="white")
        self.compute.grid(row=11, column=0, padx=5, pady=(10, 15), sticky=W)

        # BUTTON TO PLAY TRANSFORMATION SYNTHESIS OUTPUT
        self.transf_output = Button(self.parent, text=">", command=lambda: audio.play_wav(
            'output_sounds/' + os.path.basename(self.filelocation1.get())[:-4] + '_stftMorph.wav'), bg="gray30",
                                    fg="white")
        self.transf_output.grid(row=11, column=0, padx=(165, 5), pady=(10, 15), sticky=W)

        # define options for opening file
        self.file_opt = options = {}
        options['defaultextension'] = '.wav'
        options['filetypes'] = [('All files', '.*'), ('Wav files', '.wav')]
        options['initialdir'] = 'sounds/'
        options['title'] = 'Open a mono audio file .wav with sample frequency 44100 Hz'

    def browse_file1(self):

        self.filename1 = tkFileDialog.askopenfilename(**self.file_opt)

        # set the text of the self.filelocation
        self.filelocation1.delete(0, END)
        self.filelocation1.insert(0, self.filename1)

    def browse_file2(self):

        self.filename2 = tkFileDialog.askopenfilename(**self.file_opt)

        # set the text of the self.filelocation
        self.filelocation2.delete(0, END)
        self.filelocation2.insert(0, self.filename2)

    def transformation_synthesis(self):

        try:
            inputFile1 = self.filelocation1.get()
            inputFile2 = self.filelocation2.get()
            window1 = self.w1_type.get()
            window2 = self.w2_type.get()
            M1 = int(self.M1.get())
            M2 = int(self.M2.get())
            N1 = int(self.N1.get())
            N2 = int(self.N2.get())
            H1 = int(self.H1.get())
            smoothf = float(self.smoothf.get())
            balancef = float(self.balancef.get())

            sT.main(inputFile1, inputFile2, window1, window2, M1, M2, N1, N2, H1, smoothf, balancef)

        except ValueError as errorMessage:
            tkMessageBox.showerror("Input values error", errorMessage)



# GUI frame for the hpsMorph_function.py

import os
from Tkinter import *
import tkFileDialog
import tkMessageBox

import numpy as np

from smst.utils import audio
from . import hpsMorph_function as hM


class HpsMorphFrame:
    def __init__(self, parent):

        self.parent = parent
        self.initUI()

    def initUI(self):

        ## INPUT FILE 1
        choose1_label = "inputFile1:"
        Label(self.parent, text=choose1_label).grid(row=0, column=0, sticky=W, padx=5, pady=(10, 2))

        # TEXTBOX TO PRINT PATH OF THE SOUND FILE
        self.filelocation1 = Entry(self.parent)
        self.filelocation1.focus_set()
        self.filelocation1["width"] = 30
        self.filelocation1.grid(row=0, column=0, sticky=W, padx=(75, 5), pady=(10, 2))
        self.filelocation1.delete(0, END)
        self.filelocation1.insert(0, 'sounds/violin-B3.wav')

        # BUTTON TO BROWSE SOUND FILE 1
        open_file1 = Button(self.parent, text="...", command=self.browse_file1)  # see: def browse_file(self)
        open_file1.grid(row=0, column=0, sticky=W, padx=(330, 6),
                        pady=(10, 2))  # put it beside the filelocation textbox

        # BUTTON TO PREVIEW SOUND FILE 1
        preview1 = Button(self.parent, text=">", command=lambda: audio.play_wav(self.filelocation1.get()), bg="gray30",
                          fg="white")
        preview1.grid(row=0, column=0, sticky=W, padx=(375, 6), pady=(10, 2))

        # ANALYSIS WINDOW TYPE SOUND 1
        wtype1_label = "window1:"
        Label(self.parent, text=wtype1_label).grid(row=1, column=0, sticky=W, padx=5, pady=(4, 2))
        self.w1_type = StringVar()
        self.w1_type.set("blackman")  # initial value
        window1_option = OptionMenu(self.parent, self.w1_type, "rectangular", "hanning", "hamming", "blackman",
                                    "blackmanharris")
        window1_option.grid(row=1, column=0, sticky=W, padx=(68, 5), pady=(4, 2))

        # WINDOW SIZE SOUND 1
        M1_label = "M1:"
        Label(self.parent, text=M1_label).grid(row=1, column=0, sticky=W, padx=(180, 5), pady=(4, 2))
        self.M1 = Entry(self.parent, justify=CENTER)
        self.M1["width"] = 5
        self.M1.grid(row=1, column=0, sticky=W, padx=(208, 5), pady=(4, 2))
        self.M1.delete(0, END)
        self.M1.insert(0, "1001")

        # FFT SIZE SOUND 1
        N1_label = "N1:"
        Label(self.parent, text=N1_label).grid(row=1, column=0, sticky=W, padx=(265, 5), pady=(4, 2))
        self.N1 = Entry(self.parent, justify=CENTER)
        self.N1["width"] = 5
        self.N1.grid(row=1, column=0, sticky=W, padx=(290, 5), pady=(4, 2))
        self.N1.delete(0, END)
        self.N1.insert(0, "1024")

        # THRESHOLD MAGNITUDE SOUND 1
        t1_label = "t1:"
        Label(self.parent, text=t1_label).grid(row=1, column=0, sticky=W, padx=(343, 5), pady=(4, 2))
        self.t1 = Entry(self.parent, justify=CENTER)
        self.t1["width"] = 5
        self.t1.grid(row=1, column=0, sticky=W, padx=(370, 5), pady=(4, 2))
        self.t1.delete(0, END)
        self.t1.insert(0, "-100")

        # MIN DURATION SINUSOIDAL TRACKS SOUND 1
        minSineDur1_label = "minSineDur1:"
        Label(self.parent, text=minSineDur1_label).grid(row=2, column=0, sticky=W, padx=(5, 5), pady=(4, 2))
        self.minSineDur1 = Entry(self.parent, justify=CENTER)
        self.minSineDur1["width"] = 5
        self.minSineDur1.grid(row=2, column=0, sticky=W, padx=(92, 5), pady=(4, 2))
        self.minSineDur1.delete(0, END)
        self.minSineDur1.insert(0, "0.05")

        # MIN FUNDAMENTAL FREQUENCY SOUND 1
        minf01_label = "minf01:"
        Label(self.parent, text=minf01_label).grid(row=2, column=0, sticky=W, padx=(157, 5), pady=(4, 2))
        self.minf01 = Entry(self.parent, justify=CENTER)
        self.minf01["width"] = 5
        self.minf01.grid(row=2, column=0, sticky=W, padx=(208, 5), pady=(4, 2))
        self.minf01.delete(0, END)
        self.minf01.insert(0, "200")

        # MAX FUNDAMENTAL FREQUENCY SOUND 1
        maxf01_label = "maxf01:"
        Label(self.parent, text=maxf01_label).grid(row=2, column=0, sticky=W, padx=(270, 5), pady=(4, 2))
        self.maxf01 = Entry(self.parent, justify=CENTER)
        self.maxf01["width"] = 5
        self.maxf01.grid(row=2, column=0, sticky=W, padx=(325, 5), pady=(4, 2))
        self.maxf01.delete(0, END)
        self.maxf01.insert(0, "300")

        # MAX ERROR ACCEPTED SOUND 1
        f0et1_label = "f0et1:"
        Label(self.parent, text=f0et1_label).grid(row=3, column=0, sticky=W, padx=5, pady=(4, 2))
        self.f0et1 = Entry(self.parent, justify=CENTER)
        self.f0et1["width"] = 3
        self.f0et1.grid(row=3, column=0, sticky=W, padx=(45, 5), pady=(4, 2))
        self.f0et1.delete(0, END)
        self.f0et1.insert(0, "10")

        # ALLOWED DEVIATION OF HARMONIC TRACKS SOUND 1
        harmDevSlope1_label = "harmDevSlope1:"
        Label(self.parent, text=harmDevSlope1_label).grid(row=3, column=0, sticky=W, padx=(108, 5), pady=(4, 2))
        self.harmDevSlope1 = Entry(self.parent, justify=CENTER)
        self.harmDevSlope1["width"] = 5
        self.harmDevSlope1.grid(row=3, column=0, sticky=W, padx=(215, 5), pady=(4, 2))
        self.harmDevSlope1.delete(0, END)
        self.harmDevSlope1.insert(0, "0.01")

        ###
        # SEPARATION LINE
        Frame(self.parent, height=1, width=50, bg="black").grid(row=4, pady=5, sticky=W + E)
        ###

        ## INPUT FILE 2
        choose2_label = "inputFile2:"
        Label(self.parent, text=choose2_label).grid(row=5, column=0, sticky=W, padx=5, pady=(2, 2))

        # TEXTBOX TO PRINT PATH OF THE SOUND FILE
        self.filelocation2 = Entry(self.parent)
        self.filelocation2.focus_set()
        self.filelocation2["width"] = 30
        self.filelocation2.grid(row=5, column=0, sticky=W, padx=(75, 5), pady=(2, 2))
        self.filelocation2.delete(0, END)
        self.filelocation2.insert(0, 'sounds/soprano-E4.wav')

        # BUTTON TO BROWSE SOUND FILE 2
        open_file2 = Button(self.parent, text="...", command=self.browse_file2)  # see: def browse_file(self)
        open_file2.grid(row=5, column=0, sticky=W, padx=(330, 6), pady=(2, 2))  # put it beside the filelocation textbox

        # BUTTON TO PREVIEW SOUND FILE 2
        preview2 = Button(self.parent, text=">", command=lambda: audio.play_wav(self.filelocation2.get()), bg="gray30",
                          fg="white")
        preview2.grid(row=5, column=0, sticky=W, padx=(375, 6), pady=(2, 2))


        # ANALYSIS WINDOW TYPE SOUND 2
        wtype2_label = "window2:"
        Label(self.parent, text=wtype2_label).grid(row=6, column=0, sticky=W, padx=5, pady=(4, 2))
        self.w2_type = StringVar()
        self.w2_type.set("hamming")  # initial value
        window2_option = OptionMenu(self.parent, self.w2_type, "rectangular", "hanning", "hamming", "blackman",
                                    "blackmanharris")
        window2_option.grid(row=6, column=0, sticky=W, padx=(68, 5), pady=(4, 2))

        # WINDOW SIZE SOUND 2
        M2_label = "M2:"
        Label(self.parent, text=M2_label).grid(row=6, column=0, sticky=W, padx=(180, 5), pady=(4, 2))
        self.M2 = Entry(self.parent, justify=CENTER)
        self.M2["width"] = 5
        self.M2.grid(row=6, column=0, sticky=W, padx=(208, 5), pady=(4, 2))
        self.M2.delete(0, END)
        self.M2.insert(0, "901")

        # FFT SIZE SOUND 2
        N2_label = "N2:"
        Label(self.parent, text=N2_label).grid(row=6, column=0, sticky=W, padx=(265, 5), pady=(4, 2))
        self.N2 = Entry(self.parent, justify=CENTER)
        self.N2["width"] = 5
        self.N2.grid(row=6, column=0, sticky=W, padx=(290, 5), pady=(4, 2))
        self.N2.delete(0, END)
        self.N2.insert(0, "1024")

        # THRESHOLD MAGNITUDE SOUND 2
        t2_label = "t2:"
        Label(self.parent, text=t2_label).grid(row=6, column=0, sticky=W, padx=(343, 5), pady=(4, 2))
        self.t2 = Entry(self.parent, justify=CENTER)
        self.t2["width"] = 5
        self.t2.grid(row=6, column=0, sticky=W, padx=(370, 5), pady=(4, 2))
        self.t2.delete(0, END)
        self.t2.insert(0, "-100")

        # MIN DURATION SINUSOIDAL TRACKS SOUND 2
        minSineDur2_label = "minSineDur2:"
        Label(self.parent, text=minSineDur2_label).grid(row=7, column=0, sticky=W, padx=(5, 5), pady=(4, 2))
        self.minSineDur2 = Entry(self.parent, justify=CENTER)
        self.minSineDur2["width"] = 5
        self.minSineDur2.grid(row=7, column=0, sticky=W, padx=(92, 5), pady=(4, 2))
        self.minSineDur2.delete(0, END)
        self.minSineDur2.insert(0, "0.05")

        # MIN FUNDAMENTAL FREQUENCY SOUND 2
        minf02_label = "minf02:"
        Label(self.parent, text=minf02_label).grid(row=7, column=0, sticky=W, padx=(157, 5), pady=(4, 2))
        self.minf02 = Entry(self.parent, justify=CENTER)
        self.minf02["width"] = 5
        self.minf02.grid(row=7, column=0, sticky=W, padx=(208, 5), pady=(4, 2))
        self.minf02.delete(0, END)
        self.minf02.insert(0, "250")

        # MAX FUNDAMENTAL FREQUENCY SOUND 2
        maxf02_label = "maxf02:"
        Label(self.parent, text=maxf02_label).grid(row=7, column=0, sticky=W, padx=(270, 5), pady=(4, 2))
        self.maxf02 = Entry(self.parent, justify=CENTER)
        self.maxf02["width"] = 5
        self.maxf02.grid(row=7, column=0, sticky=W, padx=(325, 5), pady=(4, 2))
        self.maxf02.delete(0, END)
        self.maxf02.insert(0, "500")

        # MAX ERROR ACCEPTED SOUND 2
        f0et2_label = "f0et2:"
        Label(self.parent, text=f0et2_label).grid(row=8, column=0, sticky=W, padx=5, pady=(4, 2))
        self.f0et2 = Entry(self.parent, justify=CENTER)
        self.f0et2["width"] = 3
        self.f0et2.grid(row=8, column=0, sticky=W, padx=(45, 5), pady=(4, 2))
        self.f0et2.delete(0, END)
        self.f0et2.insert(0, "10")

        # ALLOWED DEVIATION OF HARMONIC TRACKS SOUND 2
        harmDevSlope2_label = "harmDevSlope2:"
        Label(self.parent, text=harmDevSlope2_label).grid(row=8, column=0, sticky=W, padx=(108, 5), pady=(4, 2))
        self.harmDevSlope2 = Entry(self.parent, justify=CENTER)
        self.harmDevSlope2["width"] = 5
        self.harmDevSlope2.grid(row=8, column=0, sticky=W, padx=(215, 5), pady=(4, 2))
        self.harmDevSlope2.delete(0, END)
        self.harmDevSlope2.insert(0, "0.01")

        ###
        # SEPARATION LINE
        Frame(self.parent, height=1, width=50, bg="black").grid(row=9, pady=5, sticky=W + E)
        ###

        # MAX NUMBER OF HARMONICS SOUND 1
        nH_label = "nH:"
        Label(self.parent, text=nH_label).grid(row=10, column=0, sticky=W, padx=(5, 5), pady=(2, 2))
        self.nH = Entry(self.parent, justify=CENTER)
        self.nH["width"] = 5
        self.nH.grid(row=10, column=0, sticky=W, padx=(35, 5), pady=(2, 2))
        self.nH.delete(0, END)
        self.nH.insert(0, "60")

        # DECIMATION FACTOR SOUND 1
        stocf_label = "stocf:"
        Label(self.parent, text=stocf_label).grid(row=10, column=0, sticky=W, padx=(98, 5), pady=(2, 2))
        self.stocf = Entry(self.parent, justify=CENTER)
        self.stocf["width"] = 5
        self.stocf.grid(row=10, column=0, sticky=W, padx=(138, 5), pady=(2, 2))
        self.stocf.delete(0, END)
        self.stocf.insert(0, "0.1")

        # BUTTON TO DO THE ANALYSIS OF THE SOUND
        self.compute = Button(self.parent, text="Analysis", command=self.analysis, bg="dark red", fg="white")
        self.compute.grid(row=10, column=0, padx=(210, 5), pady=(2, 2), sticky=W)

        ###
        # SEPARATION LINE
        Frame(self.parent, height=1, width=50, bg="black").grid(row=11, pady=5, sticky=W + E)
        ###

        #
        hfreqIntp_label = "harmonic frequencies interpolation factors, 0 to 1 (time,value pairs)"
        Label(self.parent, text=hfreqIntp_label).grid(row=12, column=0, sticky=W, padx=5, pady=(2, 2))
        self.hfreqIntp = Entry(self.parent, justify=CENTER)
        self.hfreqIntp["width"] = 35
        self.hfreqIntp.grid(row=13, column=0, sticky=W + E, padx=5, pady=(0, 2))
        self.hfreqIntp.delete(0, END)
        self.hfreqIntp.insert(0, "[0, 0, .1, 0, .9, 1, 1, 1]")

        #
        hmagIntp_label = "harmonic magnitudes interpolation factors, 0 to 1 (time,value pairs)"
        Label(self.parent, text=hmagIntp_label).grid(row=14, column=0, sticky=W, padx=5, pady=(5, 2))
        self.hmagIntp = Entry(self.parent, justify=CENTER)
        self.hmagIntp["width"] = 35
        self.hmagIntp.grid(row=15, column=0, sticky=W + E, padx=5, pady=(0, 2))
        self.hmagIntp.delete(0, END)
        self.hmagIntp.insert(0, "[0, 0, .1, 0, .9, 1, 1, 1]")

        #
        stocIntp_label = "stochastic component interpolation factors, 0 to 1 (time,value pairs)"
        Label(self.parent, text=stocIntp_label).grid(row=16, column=0, sticky=W, padx=5, pady=(5, 2))
        self.stocIntp = Entry(self.parent, justify=CENTER)
        self.stocIntp["width"] = 35
        self.stocIntp.grid(row=17, column=0, sticky=W + E, padx=5, pady=(0, 2))
        self.stocIntp.delete(0, END)
        self.stocIntp.insert(0, "[0, 0, .1, 0, .9, 1, 1, 1]")


        # BUTTON TO DO THE SYNTHESIS
        self.compute = Button(self.parent, text="Apply Transformation", command=self.transformation_synthesis,
                              bg="dark green", fg="white")
        self.compute.grid(row=18, column=0, padx=5, pady=(10, 15), sticky=W)

        # BUTTON TO PLAY TRANSFORMATION SYNTHESIS OUTPUT
        self.transf_output = Button(self.parent, text=">", command=lambda: audio.play_wav(
            'output_sounds/' + os.path.basename(self.filelocation1.get())[:-4] + '_hpsMorph.wav'), bg="gray30",
                                    fg="white")
        self.transf_output.grid(row=18, column=0, padx=(165, 5), pady=(10, 15), sticky=W)

        # define options for opening file
        self.file_opt = options = {}
        options['defaultextension'] = '.wav'
        options['filetypes'] = [('All files', '.*'), ('Wav files', '.wav')]
        options['initialdir'] = 'sounds/'
        options['title'] = 'Open a mono audio file .wav with sample frequency 44100 Hz'

    def browse_file1(self):

        self.filename1 = tkFileDialog.askopenfilename(**self.file_opt)

        # set the text of the self.filelocation
        self.filelocation1.delete(0, END)
        self.filelocation1.insert(0, self.filename1)

    def browse_file2(self):

        self.filename2 = tkFileDialog.askopenfilename(**self.file_opt)

        # set the text of the self.filelocation
        self.filelocation2.delete(0, END)
        self.filelocation2.insert(0, self.filename2)

    def analysis(self):

        try:
            inputFile1 = self.filelocation1.get()
            window1 = self.w1_type.get()
            M1 = int(self.M1.get())
            N1 = int(self.N1.get())
            t1 = int(self.t1.get())
            minSineDur1 = float(self.minSineDur1.get())
            minf01 = int(self.minf01.get())
            maxf01 = int(self.maxf01.get())
            f0et1 = int(self.f0et1.get())
            harmDevSlope1 = float(self.harmDevSlope1.get())

            nH = int(self.nH.get())
            stocf = float(self.stocf.get())

            inputFile2 = self.filelocation2.get()
            window2 = self.w2_type.get()
            M2 = int(self.M2.get())
            N2 = int(self.N2.get())
            t2 = int(self.t2.get())
            minSineDur2 = float(self.minSineDur2.get())
            minf02 = int(self.minf02.get())
            maxf02 = int(self.maxf02.get())
            f0et2 = int(self.f0et2.get())
            harmDevSlope2 = float(self.harmDevSlope2.get())

            self.inputFile1, self.fs1, self.hfreq1, self.hmag1, self.stocEnv1, \
            self.inputFile2, self.hfreq2, self.hmag2, self.stocEnv2 = hM.analysis(inputFile1, window1, M1, N1, t1, \
                                                                                  minSineDur1, nH, minf01, maxf01,
                                                                                  f0et1, harmDevSlope1, stocf,
                                                                                  inputFile2, window2, M2, N2, t2,
                                                                                  minSineDur2, minf02, maxf02, f0et2,
                                                                                  harmDevSlope2)

        except ValueError as errorMessage:
            tkMessageBox.showerror("Input values error", errorMessage)

    def transformation_synthesis(self):

        try:
            inputFile1 = self.inputFile1
            fs = self.fs1
            hfreq1 = self.hfreq1
            hmag1 = self.hmag1
            stocEnv1 = self.stocEnv1
            inputFile2 = self.inputFile2
            hfreq2 = self.hfreq2
            hmag2 = self.hmag2
            stocEnv2 = self.stocEnv2
            hfreqIntp = np.array(eval(self.hfreqIntp.get()))
            hmagIntp = np.array(eval(self.hmagIntp.get()))
            stocIntp = np.array(eval(self.stocIntp.get()))

            hM.transformation_synthesis(inputFile1, fs, hfreq1, hmag1, stocEnv1, inputFile2, hfreq2, hmag2, stocEnv2,
                                        hfreqIntp, hmagIntp, stocIntp)

        except ValueError as errorMessage:
            tkMessageBox.showerror("Input values error", errorMessage)

        except AttributeError:
            tkMessageBox.showerror("Analysis not computed", "First you must analyse the sound!")



# function call to the transformation functions of relevance for the hpsModel

import os

import matplotlib.pyplot as plt
import numpy as np
from scipy.signal import get_window

from smst.utils import audio, files
from smst.models import harmonic, hps
from .. import demo_sound_path


def analysis(inputFile=demo_sound_path('sax-phrase-short.wav'), window='blackman', M=601, N=1024, t=-100,
             minSineDur=0.1, nH=100, minf0=350, maxf0=700, f0et=5, harmDevSlope=0.01, stocf=0.1,
             interactive=True, plotFile=False):
    """
    Analyze a sound with the harmonic plus stochastic model
    inputFile: input sound file (monophonic with sampling rate of 44100)
    window: analysis window type (rectangular, hanning, hamming, blackman, blackmanharris)
    M: analysis window size
    N: fft size (power of two, bigger or equal than M)
    t: magnitude threshold of spectral peaks
    minSineDur: minimum duration of sinusoidal tracks
    nH: maximum number of harmonics
    minf0: minimum fundamental frequency in sound
    maxf0: maximum fundamental frequency in sound
    f0et: maximum error accepted in f0 detection algorithm
    harmDevSlope: allowed deviation of harmonic tracks, higher harmonics have higher allowed deviation
    stocf: decimation factor used for the stochastic approximation
    returns inputFile: input file name; fs: sampling rate of input file,
            hfreq, hmag: harmonic frequencies, magnitude; mYst: stochastic residual
    """

    # size of fft used in synthesis
    Ns = 512

    # hop size (has to be 1/4 of Ns)
    H = 128

    # read input sound
    (fs, x) = audio.read_wav(inputFile)

    # compute analysis window
    w = get_window(window, M)

    # compute the harmonic plus stochastic model of the whole sound
    hfreq, hmag, hphase, mYst = hps.from_audio(x, fs, w, N, H, t, nH, minf0, maxf0, f0et, harmDevSlope, minSineDur, Ns,
                                              stocf)

    # synthesize the harmonic plus stochastic model without original phases
    y, yh, yst = hps.to_audio(hfreq, hmag, np.array([]), mYst, Ns, H, fs)

    # write output sound
    outputFile = 'output_sounds/' + os.path.basename(inputFile)[:-4] + '_hpsModel.wav'
    audio.write_wav(y, fs, outputFile)

    # create figure to plot
    plt.figure(figsize=(12, 9))

    # frequency range to plot
    maxplotfreq = 15000.0

    # plot the input sound
    plt.subplot(3, 1, 1)
    plt.plot(np.arange(x.size) / float(fs), x)
    plt.axis([0, x.size / float(fs), min(x), max(x)])
    plt.ylabel('amplitude')
    plt.xlabel('time (sec)')
    plt.title('input sound: x')

    # plot spectrogram stochastic compoment
    plt.subplot(3, 1, 2)
    numFrames = int(mYst[:, 0].size)
    sizeEnv = int(mYst[0, :].size)
    frmTime = H * np.arange(numFrames) / float(fs)
    binFreq = (.5 * fs) * np.arange(sizeEnv * maxplotfreq / (.5 * fs)) / sizeEnv
    plt.pcolormesh(frmTime, binFreq, np.transpose(mYst[:, :sizeEnv * maxplotfreq / (.5 * fs) + 1]))
    plt.autoscale(tight=True)

    # plot harmonic on top of stochastic spectrogram
    if (hfreq.shape[1] > 0):
        harms = hfreq * np.less(hfreq, maxplotfreq)
        harms[harms == 0] = np.nan
        numFrames = int(harms[:, 0].size)
        frmTime = H * np.arange(numFrames) / float(fs)
        plt.plot(frmTime, harms, color='k', ms=3, alpha=1)
        plt.xlabel('time (sec)')
        plt.ylabel('frequency (Hz)')
        plt.autoscale(tight=True)
        plt.title('harmonics + stochastic spectrogram')

    # plot the output sound
    plt.subplot(3, 1, 3)
    plt.plot(np.arange(y.size) / float(fs), y)
    plt.axis([0, y.size / float(fs), min(y), max(y)])
    plt.ylabel('amplitude')
    plt.xlabel('time (sec)')
    plt.title('output sound: y')

    plt.tight_layout()

    if interactive:
        plt.show(block=False)
    if plotFile:
        plt.savefig('output_plots/%s_hps_transformation_analysis.png' % files.strip_file(inputFile))

    return inputFile, fs, hfreq, hmag, mYst


def transformation_synthesis(inputFile, fs, hfreq, hmag, mYst,
                             freqScaling=np.array([0, 1.2, 2.01, 1.2, 2.679, .7, 3.146, .7]),
                             freqStretching=np.array([0, 1, 2.01, 1, 2.679, 1.5, 3.146, 1.5]), timbrePreservation=1,
                             timeScaling=np.array([0, 0, 2.138, 2.138 - 1.0, 3.146, 3.146]),
                             interactive=True, plotFile=False):
    """
    transform the analysis values returned by the analysis function and synthesize the sound
    inputFile: name of input file
    fs: sampling rate of input file
    hfreq, hmag: harmonic frequencies and magnitudes
    mYst: stochastic residual
    freqScaling: frequency scaling factors, in time-value pairs (value of 1 no scaling)
    freqStretching: frequency stretching factors, in time-value pairs (value of 1 no stretching)
    timbrePreservation: 1 preserves original timbre, 0 it does not
    timeScaling: time scaling factors, in time-value pairs
    """

    # size of fft used in synthesis
    Ns = 512

    # hop size (has to be 1/4 of Ns)
    H = 128

    # frequency scaling of the harmonics
    hfreqt, hmagt = harmonic.scale_frequencies(hfreq, hmag, freqScaling, freqStretching, timbrePreservation, fs)

    # time scaling the sound
    yhfreq, yhmag, ystocEnv = hps.scale_time(hfreqt, hmagt, mYst, timeScaling)

    # synthesis from the trasformed hps representation
    y, yh, yst = hps.to_audio(yhfreq, yhmag, np.array([]), ystocEnv, Ns, H, fs)

    # write output sound
    outputFile = 'output_sounds/' + os.path.basename(inputFile)[:-4] + '_hpsModelTransformation.wav'
    audio.write_wav(y, fs, outputFile)

    # create figure to plot
    plt.figure(figsize=(12, 6))

    # frequency range to plot
    maxplotfreq = 15000.0

    # plot spectrogram of transformed stochastic compoment
    plt.subplot(2, 1, 1)
    numFrames = int(ystocEnv[:, 0].size)
    sizeEnv = int(ystocEnv[0, :].size)
    frmTime = H * np.arange(numFrames) / float(fs)
    binFreq = (.5 * fs) * np.arange(sizeEnv * maxplotfreq / (.5 * fs)) / sizeEnv
    plt.pcolormesh(frmTime, binFreq, np.transpose(ystocEnv[:, :sizeEnv * maxplotfreq / (.5 * fs) + 1]))
    plt.autoscale(tight=True)

    # plot transformed harmonic on top of stochastic spectrogram
    if (yhfreq.shape[1] > 0):
        harms = yhfreq * np.less(yhfreq, maxplotfreq)
        harms[harms == 0] = np.nan
        numFrames = int(harms[:, 0].size)
        frmTime = H * np.arange(numFrames) / float(fs)
        plt.plot(frmTime, harms, color='k', ms=3, alpha=1)
        plt.xlabel('time (sec)')
        plt.ylabel('frequency (Hz)')
        plt.autoscale(tight=True)
        plt.title('harmonics + stochastic spectrogram')

    # plot the output sound
    plt.subplot(2, 1, 2)
    plt.plot(np.arange(y.size) / float(fs), y)
    plt.axis([0, y.size / float(fs), min(y), max(y)])
    plt.ylabel('amplitude')
    plt.xlabel('time (sec)')
    plt.title('output sound: y')

    plt.tight_layout()

    if interactive:
        plt.show()
    if plotFile:
        plt.savefig('output_plots/%s_hps_transformation_synthesis.png' % files.strip_file(inputFile))


def main(interactive=True, plotFile=False):
    # analysis
    inputFile, fs, hfreq, hmag, mYst = analysis(interactive=interactive, plotFile=plotFile)

    # transformation and synthesis
    transformation_synthesis(inputFile, fs, hfreq, hmag, mYst, interactive=interactive, plotFile=plotFile)


if __name__ == "__main__":
    main()
